from .connect import connect_node
from pyfiglet import Figlet
from .menu import menu
import os

fig = Figlet(font='slant')

def cli():
    os.system('cls' if os.name=='nt' else 'clear')
    print("\n")
    print(fig.renderText('jutsu'))
    print("\n")
    n = connect_node()
    print("\n")
    menu(n)


from PyInquirer import prompt
from oogway import Node

node_rpc = [
    {
        'type': 'input',
        'name': 'host',
        'message': 'Enter your node host'
    },
    {
        'type': 'input',
        'name': 'port',
        'default': '8332',
        'message': 'Enter your node port'
    },
    {
        'type': 'input',
        'name': 'username',
        'message': 'Enter your bitcoind RPC username'
    },
    {
        'type': 'password',
        'name': 'password',
        'message': 'Enter your bitcoind RPC password'
    },
]

def connect_node():
    answers = prompt(node_rpc)
    url = "http://%s:%s@%s:%s" % (answers['username'], answers['password'], answers['host'], answers['port'])
    n = Node(url, timeout=2000)
    return n


import colorama
from yaspin import yaspin
import time
import datetime
from .exit_func import exit_func

colorama.init()

@yaspin(text="Calculating...", color="green")
def current_height(n):
    info = n.getblockchaininfo()
    return int(info['headers'])


def next_halving(n):
    start = time.time()

    ch = current_height(n)
    next_halving_height = 210000 + 630000
    remaining_blocks = next_halving_height - ch

    remaining_time_sec = remaining_blocks * 10 * 60

    now = time.time()
    then = now + remaining_time_sec

    halving_date = datetime.datetime.fromtimestamp(then).strftime('%Y-%m-%d %H:%M')
    remaining_time_days = int(remaining_time_sec / 3600 / 24)

    end = time.time()
    print("\n")
    print(colorama.Fore.WHITE + "Next halving:              "+ colorama.Fore.GREEN + str(halving_date) + colorama.Fore.WHITE + "")
    print(colorama.Fore.WHITE + "Time remaining:            "+ colorama.Fore.GREEN + str(remaining_time_days) + colorama.Fore.WHITE + " days")
    print(colorama.Fore.WHITE + "Next block reward:         "+ colorama.Fore.GREEN + str(3.125) + colorama.Fore.WHITE + " BTC")
    print("\n")
    t = (end - start)
    seconds = int(t)
    print("Time elapsed: %d seconds" % seconds)
    print("\n")
    exit_func(n)


import colorama
from yaspin import yaspin
import time
from .exit_func import exit_func

colorama.init()

@yaspin(text="Calculating...", color="green")
def current_size(n):
    info = n.getblockchaininfo()
    return float(info['size_on_disk'])


def chain_size(n):
    start = time.time()

    cs = current_size(n)
    cs = cs / 1e9
    cs = "%.2f" % cs

    end = time.time()
    print("\n")
    print(colorama.Fore.WHITE + "Current blockchain size:              "+ colorama.Fore.GREEN + str(cs) + colorama.Fore.WHITE + " GB")
    print("\n")
    t = (end - start)
    seconds = int(t)
    print("Time elapsed: %d seconds" % seconds)
    print("\n")
    exit_func(n)


import colorama
from yaspin import yaspin
import time
from .exit_func import exit_func

colorama.init()

@yaspin(text="Verifying...", color="green")
def calculate_circulating(n):
    """returns total amount of Bitcoin in the UTXO set"""
    c = n.gettxoutsetinfo()
    return c['total_amount'], c['height']

def calculate_max():
    """calculates the maximum supply based on btcdirect.eu/en-gb/how-many-bitcoin"""
    summation = 0
    for i in range(0, 33):
        summation += 210000 * ((50*(10**8)) / 2**i)
    
    summation = summation / (10**8)
    return summation

def circ_is_valid(c, maximum):
    """returns true if circulating supply is valid and inferior to total supply"""
    if c < maximum:
        v = True
    else:
        v = False
    return v

def supply(n):
    start = time.time()
    c = calculate_circulating(n)
    maximum = calculate_max()
    valid = circ_is_valid(c[0], maximum)
    calc_sup = (209999*50)+(210000*25)+(210000*12.5)+((int(c[1])-630000)*6.25)
    end = time.time()
    print("\n")
    print(colorama.Fore.WHITE + "Circulating supply:   "+ colorama.Fore.GREEN + str(c[0]) + colorama.Fore.WHITE + " BTC")
    print(colorama.Fore.WHITE + "Calculated supply:   ~"+ colorama.Fore.GREEN + str(calc_sup) + colorama.Fore.WHITE + " BTC")
    print(colorama.Fore.WHITE + "Maximum supply:       "+ colorama.Fore.GREEN + str(maximum) + colorama.Fore.WHITE + " BTC")
    print(colorama.Fore.WHITE + "Valid supply:         "+ colorama.Fore.GREEN + str(valid) + colorama.Fore.WHITE + "")
    print("\n")
    t = (end - start)
    minutes = t // 60
    t %= 60
    seconds = t
    print("Time elapsed: %d minutes and %d seconds" % (minutes, seconds))
    print("\n")
    exit_func(n)



from PyInquirer import prompt
from .supply import supply
from .block_height import block_height
from .halving import next_halving
from .chain_size import chain_size

menu_items = [
    {
        'type': 'list',
        'name': 'menu',
        'message': 'What do you want to verify?',
        'choices': [
            'Circulating and maximum supply',
            'Block height',
            'Next halving',
            'Blockchain size',
            'Exit'
        ]
    }
]

confirm = [
    {
        'type': 'confirm',
        'message': 'This operation will take a few minutes. Continue?',
        'name': 'continue',
        'default': True,
    }
]

def menu(n):
    answers = prompt(menu_items)
    if answers['menu'] == "Circulating and maximum supply":
        confirmation = prompt(confirm)
        if confirmation['continue'] == True:
            supply(n)
            menu(n)
        else:
            menu(n)

    elif answers['menu'] == "Block height":
        block_height(n)
        menu(n)
    
    elif answers['menu'] == "Next halving":
        next_halving(n)
        menu(n)
    
    elif answers['menu'] == "Blockchain size":
        chain_size(n)
        menu(n)
    
    elif answers['menu'] == "Exit":
        exit()


import setuptools
from jutsu import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
        name="jutsu",
        version=__version__,
        author="Merwane Drai",
        author_email="merwane@6conf.com",
        description="Independently verify the Bitcoin supply",
        long_description = long_description,
        long_description_content_type = "text/markdown",
        url = "https://github.com/merwane/jutsu",
        download_url = "https://github.com/merwane/jutsu",
        license='MIT',
        keywords={
            "bitcoin",
            "bitcoin-verification",
            "bitcoin-cli"
            },
        classifiers=(
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Natural Language :: English',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8'
            ),
        install_requires=("oogway", "PyInquirer", "pyfiglet", "colorama", "yaspin"),
        tests_require=["pytest"],
        packages = setuptools.find_packages(),
        entry_points='''
            [console_scripts]
            jutsu=jutsu.cli:cli
        ''',
        include_package_data = True
        )


import colorama
from yaspin import yaspin
import time
from .exit_func import exit_func

colorama.init()

@yaspin(text="Verifying...", color="green")
def current_height(n):
    c = n.getblockchaininfo()
    return int(c['headers'])

def maximum_rewarded_height():
    m = (210000 * 33) - 1
    return m

def block_height(n):
    start = time.time()
    c = current_height(n)
    m = maximum_rewarded_height()
    p = int((c*100) / m)
    end = time.time()
    print("\n")
    print(colorama.Fore.WHITE + "Current block height:              "+ colorama.Fore.GREEN + str(c) + colorama.Fore.WHITE + " Blocks")
    print(colorama.Fore.WHITE + "Maximum rewardable block height:   "+ colorama.Fore.GREEN + str(m) + colorama.Fore.WHITE + " Blocks")
    print(colorama.Fore.WHITE + "Progress:                          "+ colorama.Fore.GREEN + str(p) + "%" + colorama.Fore.WHITE + " of rewardable blocks discovered")
    print("\n")
    t = (end - start)
    seconds = int(t)
    print("Time elapsed: %d seconds" % seconds)
    print("\n")
    exit_func(n)